unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  FileCtrl, StdCtrls, ExtCtrls, ComCtrls;

type
  TMainForm = class(TForm)
    FileGroup: TGroupBox;
    MainDriveCombo: TDriveComboBox;
    MainDirList: TDirectoryListBox;
    MainFileList: TFileListBox;
    MainFileEdit: TEdit;
    FileSplitter: TSplitter;
    MainPanel: TPanel;
    Panel2: TPanel;
    GroupBox1: TGroupBox;
    BoardRangeCombo: TComboBox;
    HzRadios: TRadioGroup;
    ColRangeGroup: TGroupBox;
    LoadButton: TButton;
    PlayButton: TButton;
    FileLabel: TLabel;
    PlayReportLabel: TLabel;
    tbHz: TTrackBar;
    lblHz: TLabel;
    cbForever: TCheckBox;
    cbNumDACs: TComboBox;
    Label1: TLabel;
    GroupBox2: TGroupBox;
    CountDataMemo: TMemo;
    FileDataMemo: TMemo;
    Splitter1: TSplitter;
    Panel1: TPanel;
    cbDisplayBuffer: TCheckBox;
    cbDisplayFile: TCheckBox;
    cbPadRaw: TCheckBox;
    BufDepthButton: TButton;
    BufDepth256Button: TButton;
    BufDepth1Button: TButton;
    procedure FormCreate(Sender: TObject);
    procedure BoardRangeComboChange(Sender: TObject);
    procedure PlayButtonClick(Sender: TObject);
    procedure LoadButtonClick(Sender: TObject);
    procedure MainFileListDblClick(Sender: TObject);
    procedure HzRadiosClick(Sender: TObject);
    procedure tbHzChange(Sender: TObject);
    procedure Splitter1Moved(Sender: TObject);
    procedure BufDepthButtonClick(Sender: TObject);
    procedure cbForeverClick(Sender: TObject);
    procedure BufDepth256ButtonClick(Sender: TObject);
    procedure BufDepth1ButtonClick(Sender: TObject);
  protected
    ColRange: array of record
      ColLabel: TLabel;
      RangeCombo: TComboBox;
    end;
    ClockHz: Double;
    DACCount: Integer;
    DataFile: TStringList;

    procedure SetColRangeCombos(ColHeadsCount: Integer; ColHeads: TStringList = nil);
    procedure CheckPlayEnabled;
    { Private declarations }
  public
    procedure PostCreate;
    { Public declarations }
  end;

var
  MainForm: TMainForm;





implementation

{$R *.DFM}

uses
  Math,
  AIOUSB;

procedure DumpStringToFile(S, Path: AnsiString);
var
  Fil: TFileStream;

begin
  Fil := TFileStream.Create(Path, fmCreate or fmShareDenyWrite);
  try
    Fil.Write(S[1], Length(S));
  finally
    Fil.Free;
  end;
end;

function DumpFileToString(Path: AnsiString): AnsiString;
var
  Fil: TFileStream;
  L: Integer;

begin
  Fil := TFileStream.Create(Path, fmOpenRead or fmShareDenyNone);
  try
    L := Fil.Size;
    SetString(Result, nil, L);
    Fil.Read(Result[1], L);
  finally
    Fil.Free;
  end;
end;

function InStrC(C: Char; S: AnsiString; StartIndex: Integer = 1): LongWord;
var
  I: Integer;

begin
  for I := StartIndex to Length(S) do
    if S[I] = C then begin
      Result := I;
      Exit;
    end
  ;

  Result := 0;
end;

function StringReplaceC(const S: AnsiString; const OldChar, NewChar: Char): AnsiString;
var
  I: Integer;

begin
  Result := S;
  for I := 1 to Length(Result) do
    if Result[I] = OldChar then
      Result[I] := NewChar
  ;
end;

function Tokenize(S: String; Token: Char): TStringList;
var
  I: Integer;

begin
  try
    Result := TStringList.Create;
    repeat
      I := Pos(Token, S);
      if I = 0 then begin
        Result.Add(S);
        Exit;
      end;
      Result.Add(Copy(S, 1, I - 1));
      S := Copy(S, I + 1, MAXINT);
    until False;
  except
    Result := nil;
  end;
end;

procedure LoadDefaultFont(IntoFont: TFont);
var
  FontSize: Integer;
  NCM: TNonClientMetrics;

begin
  NCM.cbSize := SizeOf(NCM);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, SizeOf(NCM), @NCM, 0) then begin
    FontSize := Round(-3/4 * NCM.lfMessageFont.lfHeight);
    IntoFont.Name := NCM.lfMessageFont.lfFaceName;
  end
  else begin
    FontSize := Abs(IntoFont.Size);
    IntoFont.Name := 'Tahoma';
  end;
  IntoFont.PixelsPerInch := 72;
  IntoFont.Size := FontSize;
end;



const
  DeviceIndex = 0;

function ValF(S: AnsiString; var Code: Integer): Double;
begin
  Val(S, Result, Code);
end;

function InW(F: Double): Word;
begin
  if F <= $0000 then
    Result := $0000
  else if F >= $FFFF then
    Result := $FFFF
  else
    Result := Round(F)
  ;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  LoadDefaultFont(Font);
end;

procedure TMainForm.PostCreate;
begin
  cbNumDACs.ItemIndex := 0;
  BoardRangeCombo.ItemIndex := 3;
end;

procedure TMainForm.BoardRangeComboChange(Sender: TObject);
begin
  CheckPlayEnabled;
end;

procedure TMainForm.CheckPlayEnabled;
begin
  PlayButton.Enabled := (BoardRangeCombo.Itemindex >= 0 )
                         and (DataFile <> nil)
  ;
  cbForever.Enabled := PlayButton.Enabled;

end;

procedure TMainForm.LoadButtonClick(Sender: TObject);
var
  FN, Content: AnsiString;
  Contents: array of TStringList;
  TempFile: TStringList;
  iRow, iCol, W: Integer;
  bGotData: Boolean;
  Code: Integer;

begin
  if MainFileEdit.Text <> '*.tsv' then

  try
    FreeAndNil(DataFile);
    FileLabel.Caption := '';

    FN := MainFileEdit.Text;
    Content := DumpFileToString(FN);
    if InStrC(#9, Content) = 0 then
      Content := StringReplaceC(Content, ',', #9)
    ;

    TempFile := TStringList.Create;
    TempFile.Text := Content;

    if cbDisplayFile.checked then begin
      filedatamemo.Text := tempfile.text;
    end;

    SetLength(Contents, TempFile.Count);
    W := 0;
    for iRow := 0 to High(Contents) do begin
      Contents[iRow] := Tokenize(TempFile[iRow], #9);
      W := Max(W, Contents[iRow].Count);
    end;
    FreeAndNil(TempFile);

    for iCol := W-1 downto 0 do begin
      bGotData := False;
      for iRow := 0 to High(Contents) do
        if Contents[iRow].Count > iCol then
          if Contents[iRow][iCol] <> '' then begin
            ValF(Contents[iRow][iCol], Code);
            if Code = 0 then begin
              bGotData := True;
              Break;
            end
            else begin
              Contents[iRow][iCol] := '';
            end;
          end
      ;
      if not bGotData then begin
        Dec(W);
        for iRow := 0 to High(Contents) do
          if Contents[iRow].Count > iCol then
            Contents[iRow].Delete(iCol)
        ;
      end;
    end;

    DataFile := TStringList.Create;
    for iRow := 0 to High(Contents) do
      DataFile.Add(Contents[iRow].Text)
    ;

    FileLabel.Caption := ExtractFileName(FN) + ' - ' + IntToStr(DataFile.Count) + ' Rows, ' + IntToStr(Contents[0].Count) + ' Cols';

    for iRow := 0 to High(Contents) do
      Contents[iRow].Free
    ;
    SetLength(Contents, 0);

    SetColRangeCombos(W);
  except
    FreeAndNil(DataFile);
  end;
  DACCount := W;
  CheckPlayEnabled;
end;

procedure TMainForm.SetColRangeCombos(ColHeadsCount: Integer; ColHeads: TStringList = nil);
var
  I, oL: Integer;

begin
  oL := Length(ColRange);
  if oL > ColHeadsCount then begin
    for I := ColHeadsCount to oL-1 do begin
      FreeAndNil(ColRange[I].ColLabel);
      FreeAndNil(ColRange[I].RangeCombo);
    end;
  end;
  SetLength(ColRange, ColHeadsCount);
  if oL < ColHeadsCount then begin
    for I := oL to ColHeadsCount-1 do with ColRange[I] do begin
      ColLabel := nil;
      ColLabel := TLabel.Create(Self);
      ColLabel.AutoSize := False;
      ColLabel.Alignment := taCenter;
      if ColHeads = nil then
        ColLabel.Caption := 'Ch ' + IntToStr(I+1)
      else
        ColLabel.Caption := ColHeads[I]
      ;
      ColLabel.SetBounds(8 + 72*I, 16, 64, 16);
      ColLabel.Parent := ColRangeGroup;
      RangeCombo := TComboBox.Create(Self);
      RangeCombo.Style := csDropDownList;
      RangeCombo.SetBounds(8 + 72*I, 32, 64, 21);
      RangeCombo.Parent := ColRangeGroup;
      RangeCombo.Items.Text := BoardRangeCombo.Items.Text; //'0-5V'#13#10'5V'#13#10'0-10V'#13#10'10V';
      RangeCombo.ItemIndex := Max(0, BoardRangeCombo.ItemIndex);
    end;
  end;
end;

procedure TMainForm.PlayButtonClick(Sender: TObject);
const
  MAXLASTDAC = 15;
var
  SampleData: array of Word;
  LastSample: array[0..MAXLASTDAC] of Word;
  iRow, iCol, iSample: Integer;
  ThisRow: TStringList;
  S, Ct: Double;
  Status: LongWord;
  ErrText, ErrCaption: AnsiString;
  RangeCode: Byte;
  countsPerDAC:String;
  I: Integer;
  LASTDAC: Integer;
  rawfile: TStringList;
  ptr:word;
  loopcount:word;
begin
  ClockHz := tbHz.Position;

  AIOUSB.AIOUSB_ReloadDeviceLinks;

  I := BoardRangeCombo.ItemIndex;
  Status := AIOUSB.DACSetBoardRange(DeviceIndex, I);
  if Status <> ERROR_SUCCESS then begin
    ErrText := 'Error ' + IntToStr(Status) + ' setting board range.';
    ErrCaption := 'DACSetBoardRange Error ' + IntToStr(Status);
    MessageBox(0, PAnsiChar(ErrText), PAnsiChar(ErrCaption), MB_ICONEXCLAMATION);
    Exit;
  end;

  rawFile := TStringList.Create;
  try
    rawFile.Capacity := DataFile.Count + 1;

    I := 0;
    LASTDAC := cbNumDACs.ItemIndex;
    GenericVendorWrite(DeviceIndex, $C7, $0001 + ((LASTDAC+1) shl 8), 0, I, nil);  // DAC Set Channel Count

    DACOutputSetCount(DeviceIndex, LASTDAC+1);

    CountDataMemo.Lines.Clear;

    PlayReportLabel.Caption := 'Parsing file';
    PlayReportLabel.Repaint;
    if (BoardRangeCombo.ItemIndex and 1) <> 0 then
      LastSample[0] := $8000
    else
      LastSample[0] := $0000
    ;
    for iCol := 1 to LASTDAC do
      LastSample[iCol] := LastSample[0]
    ;
    SetLength(SampleData, (LASTDAC+1) * DataFile.Count);
    iSample := 0;
    ThisRow := TStringList.Create;
    for iRow := 0 to DataFile.Count-1 do begin
      countsPerDAC := '';
      ThisRow.Text := DataFile[iRow];
      for iCol := 0 to LASTDAC do begin
        if (iCol < ThisRow.Count) and (ThisRow[iCol] <> '') then begin
          RangeCode := ColRange[iCol].RangeCombo.ItemIndex;
          S := StrToFloat(ThisRow[iCol]) / 5;
          if (RangeCode and $2) <> 0 then S := S / 2;
          if (RangeCode and $1) <> 0 then S := (S + 1)/2;
          Ct := $10000 * S;
          SampleData[iSample] := InW(Ct);
          countsPerDAC := countsPerDAC + IntToHex(SampleData[iSample], 4) + ' ';
          LastSample[iCol] := SampleData[iSample];
        end
        else begin
          SampleData[iSample] := LastSample[iCol];
          SampleData[iSample] := SampleData[iSample-1];
          countsPerDAC := countsPerDAC + IntToHex(SampleData[iSample], 4) + ' ';
        end;
        Inc(iSample);
      end;
      rawFile.Add(countsPerDAC);
    end;
    ThisRow.Free;

    if cbDisplayBuffer.Checked then CountDataMemo.lines.Text := rawFile.Text;

    PlayReportLabel.Caption := 'Playing ' + FloatToStr(DataFile.Count / ClockHz) + '-sec file';
    PlayReportLabel.Repaint;
    ClockHz := ClockHz;
    loopcount := 0;

    repeat
      // Status := DACOutputProcess(DeviceIndex, ClockHz, Length(SampleData), @SampleData[0]);
      Status := DACOutputProcess(DeviceIndex, ClockHz, 20 * (LASTDAC+1), @SampleData[loopcount * 20 * (LASTDAC+1)]);
      if Status <> Error_Success then begin
        Caption := IntToStr(Status) + ' error on DACOutputProcess';
        cbForever.Checked := False;
      end;
      loopcount := loopcount + 1;
      if (loopcount+1) * 20 * (LASTDAC+1) > Length(SampleData) then loopcount := 0;

      Application.ProcessMessages;
    until cbForever.Checked = FALSE;


    PlayReportLabel.Caption := '';
    PlayReportLabel.Repaint;
    if Status <> ERROR_SUCCESS then begin
      ErrText := 'Error ' + IntToStr(Status) + ' launching DAC output process.';
      ErrCaption := 'DACOutputProcess Error ' + IntToStr(Status);
      MessageBox(0, PAnsiChar(ErrText), PAnsiChar(ErrCaption), MB_ICONEXCLAMATION);
    end;

  finally
    rawFile.Free;
  end
end;

procedure TMainForm.MainFileListDblClick(Sender: TObject);
begin
  LoadButton.Click;
end;

procedure TMainForm.HzRadiosClick(Sender: TObject);
begin
  case HzRadios.ItemIndex of
    0: ClockHz := 512;
    1: ClockHz := 1024;
    2: ClockHz := 4096;
    3: ClockHz := 1000;
    4: ClockHz := 2000;
    else Exit;
  end;
  tbHz.Position := Round(ClockHz);
end;

procedure TMainForm.tbHzChange(Sender: TObject);
begin
  ClockHz := tbHz.Position;
  lblHz.Caption := IntToStr(Round(ClockHz)) + ' Hz';
end;

procedure TMainForm.Splitter1Moved(Sender: TObject);
begin
  cbDisplayFile.Left := splitter1.left;
end;

procedure TMainForm.BufDepthButtonClick(Sender: TObject);
begin
  // configure ring buffer depth to be "(ValueLSB+1)*number of channels to stream"
  GenericVendorWrite(DeviceIndex, $C7, 19, 1, 0, nil);
end;

procedure TMainForm.BufDepth256ButtonClick(Sender: TObject);
begin
  // configure ring buffer depth to be "(ValueLSB+1)*number of channels to stream"
  GenericVendorWrite(DeviceIndex, $C7, 255, 1, 0, nil);
end;

procedure TMainForm.cbForeverClick(Sender: TObject);
begin
  if cbForever.Checked then PlayButton.Click; 
end;

procedure TMainForm.BufDepth1ButtonClick(Sender: TObject);
begin
  GenericVendorWrite(DeviceIndex, $C7, 1, 1, 0, nil);
end;

end.
